"""Tests for chunking strategies."""

import pytest
from socratic_rag.chunking import FixedSizeChunker
from socratic_rag.exceptions import ChunkingError


class TestFixedSizeChunker:
    """Test FixedSizeChunker."""

    def test_chunker_initialization(self):
        """Test chunker initialization."""
        chunker = FixedSizeChunker(chunk_size=100, overlap=10)
        assert chunker.chunk_size == 100
        assert chunker.overlap == 10

    def test_chunk_text(self):
        """Test chunking text."""
        chunker = FixedSizeChunker(chunk_size=50, overlap=10)
        text = "This is a test. " * 20  # ~320 characters
        chunks = chunker.chunk(text, document_id="doc1")

        assert len(chunks) > 1
        assert all(chunk.document_id == "doc1" for chunk in chunks)
        assert all(chunk.text for chunk in chunks)

    def test_chunk_overlap(self):
        """Test that chunks have overlap."""
        chunker = FixedSizeChunker(chunk_size=50, overlap=10)
        text = "This is a test sentence. " * 10
        chunks = chunker.chunk(text, document_id="doc1")

        if len(chunks) > 1:
            # Check for overlap between consecutive chunks
            chunk1_end = chunks[0].text[-10:]
            chunk2_start = chunks[1].text[:10]
            # There should be some overlap
            assert len(chunks[0].text) <= 50

    def test_chunk_ids(self):
        """Test that chunks have unique IDs."""
        chunker = FixedSizeChunker(chunk_size=50, overlap=10)
        text = "This is a test. " * 20
        chunks = chunker.chunk(text, document_id="doc1")

        chunk_ids = [chunk.chunk_id for chunk in chunks]
        assert len(chunk_ids) == len(set(chunk_ids))  # All unique

    def test_empty_text(self):
        """Test that empty text raises error."""
        chunker = FixedSizeChunker()
        with pytest.raises(ChunkingError):
            chunker.chunk("", document_id="doc1")

    def test_invalid_chunk_size(self):
        """Test that invalid chunk_size raises error."""
        with pytest.raises(ChunkingError):
            FixedSizeChunker(chunk_size=-1)

    def test_invalid_overlap(self):
        """Test that invalid overlap raises error."""
        with pytest.raises(ChunkingError):
            FixedSizeChunker(chunk_size=100, overlap=100)

    def test_chunk_positions(self):
        """Test that chunk positions are accurate."""
        chunker = FixedSizeChunker(chunk_size=50, overlap=0)
        text = "a" * 150
        chunks = chunker.chunk(text, document_id="doc1")

        assert chunks[0].start_char == 0
        assert chunks[0].end_char == 50
        assert chunks[1].start_char == 50
        assert chunks[1].end_char == 100

    def test_short_text(self):
        """Test chunking short text."""
        chunker = FixedSizeChunker(chunk_size=100)
        text = "Short text"
        chunks = chunker.chunk(text, document_id="doc1")

        assert len(chunks) == 1
        assert chunks[0].text == text

    def test_chunk_with_metadata(self):
        """Test chunking with metadata."""
        chunker = FixedSizeChunker()
        text = "Test text. " * 100
        metadata = {"key": "value", "source": "test.txt"}
        chunks = chunker.chunk(text, document_id="doc1", metadata=metadata)

        assert all(chunk.metadata == metadata for chunk in chunks)



"""Socratic RAG - Production-grade Retrieval-Augmented Generation."""

from .client import RAGClient
from .async_client import AsyncRAGClient
from .llm_rag import LLMPoweredRAG
from .models import Chunk, Document, RAGConfig, SearchResult
from .exceptions import (
    AsyncRAGError,
    ChunkingError,
    ConfigurationError,
    DocumentNotFoundError,
    EmbeddingError,
    InvalidProviderError,
    ProcessorError,
    ProviderNotFoundError,
    SocraticRAGError,
    VectorStoreError,
)
from .embeddings import BaseEmbedder, SentenceTransformersEmbedder
from .chunking import BaseChunker, FixedSizeChunker
from .vector_stores import BaseVectorStore, ChromaDBVectorStore, QdrantVectorStore, FAISSVectorStore

__version__ = "0.1.0"

__all__ = [
    "RAGClient",
    "AsyncRAGClient",
    "LLMPoweredRAG",
    "Chunk",
    "Document",
    "RAGConfig",
    "SearchResult",
    "SocraticRAGError",
    "ConfigurationError",
    "VectorStoreError",
    "EmbeddingError",
    "ChunkingError",
    "ProcessorError",
    "DocumentNotFoundError",
    "ProviderNotFoundError",
    "InvalidProviderError",
    "AsyncRAGError",
    "BaseEmbedder",
    "SentenceTransformersEmbedder",
    "BaseChunker",
    "FixedSizeChunker",
    "BaseVectorStore",
    "ChromaDBVectorStore",
    "QdrantVectorStore",
    "FAISSVectorStore",
]



"""Main RAG client interface."""

from typing import Any, Dict, List, Optional
from .models import Document, RAGConfig, SearchResult
from .embeddings.base import BaseEmbedder
from .embeddings.sentence_transformers import SentenceTransformersEmbedder
from .chunking.base import BaseChunker
from .chunking.fixed_size import FixedSizeChunker
from .vector_stores.base import BaseVectorStore
from .vector_stores.chromadb import ChromaDBVectorStore
from .vector_stores.qdrant import QdrantVectorStore
from .vector_stores.faiss import FAISSVectorStore
from .exceptions import ConfigurationError, ProviderNotFoundError


class RAGClient:
    """Main RAG client interface.

    Provides a unified interface for document processing, embedding,
    chunking, and vector store operations.
    """

    def __init__(self, config: Optional[RAGConfig] = None) -> None:
        """Initialize RAG client.

        Args:
            config: RAGConfig object. If None, uses default configuration.

        Raises:
            ConfigurationError: If configuration is invalid.
        """
        self.config = config or RAGConfig()
        self._vector_store: Optional[BaseVectorStore] = None
        self._embedder: Optional[BaseEmbedder] = None
        self._chunker: Optional[BaseChunker] = None

    @property
    def vector_store(self) -> BaseVectorStore:
        """Lazy initialization of vector store.

        Returns:
            Configured vector store instance.

        Raises:
            ProviderNotFoundError: If vector store provider is not found.
        """
        if self._vector_store is None:
            self._vector_store = self._create_vector_store()
        return self._vector_store

    @property
    def embedder(self) -> BaseEmbedder:
        """Lazy initialization of embedder.

        Returns:
            Configured embedder instance.

        Raises:
            ProviderNotFoundError: If embedder provider is not found.
        """
        if self._embedder is None:
            self._embedder = self._create_embedder()
        return self._embedder

    @property
    def chunker(self) -> BaseChunker:
        """Lazy initialization of chunker.

        Returns:
            Configured chunker instance.

        Raises:
            ProviderNotFoundError: If chunker provider is not found.
        """
        if self._chunker is None:
            self._chunker = self._create_chunker()
        return self._chunker

    def _create_vector_store(self) -> BaseVectorStore:
        """Create vector store based on configuration.

        Returns:
            Vector store instance.

        Raises:
            ProviderNotFoundError: If provider is not found.
        """
        if self.config.vector_store == "chromadb":
            return ChromaDBVectorStore(
                collection_name=self.config.collection_name,
            )
        elif self.config.vector_store == "qdrant":
            return QdrantVectorStore(
                collection_name=self.config.collection_name,
            )
        elif self.config.vector_store == "faiss":
            return FAISSVectorStore(
                collection_name=self.config.collection_name,
            )
        else:
            raise ProviderNotFoundError(
                f"Vector store provider '{self.config.vector_store}' not found. "
                f"Available: chromadb, qdrant, faiss"
            )

    def _create_embedder(self) -> BaseEmbedder:
        """Create embedder based on configuration.

        Returns:
            Embedder instance.

        Raises:
            ProviderNotFoundError: If provider is not found.
        """
        if self.config.embedder == "sentence-transformers":
            return SentenceTransformersEmbedder()
        else:
            raise ProviderNotFoundError(
                f"Embedder provider '{self.config.embedder}' not found. "
                f"Available: sentence-transformers"
            )

    def _create_chunker(self) -> BaseChunker:
        """Create chunker based on configuration.

        Returns:
            Chunker instance.

        Raises:
            ProviderNotFoundError: If provider is not found.
        """
        if self.config.chunking_strategy == "fixed":
            return FixedSizeChunker(
                chunk_size=self.config.chunk_size,
                overlap=self.config.chunk_overlap,
            )
        else:
            raise ProviderNotFoundError(
                f"Chunking strategy '{self.config.chunking_strategy}' not found. "
                f"Available: fixed"
            )

    def add_document(
        self,
        content: str,
        source: str,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> str:
        """Add document to knowledge base.

        Chunks the document, generates embeddings, and stores in vector database.

        Args:
            content: Document content.
            source: Document source (e.g., filename, URL).
            metadata: Optional metadata to attach to document.

        Returns:
            Document ID.

        Raises:
            Various exceptions for processing failures.
        """
        # Create document
        doc = Document.create(
            content=content,
            source=source,
            metadata=metadata,
        )

        # Chunk document
        chunks = self.chunker.chunk(
            text=content,
            document_id=doc.document_id,
            metadata=metadata,
        )

        # Generate embeddings
        texts = [chunk.text for chunk in chunks]
        embeddings = self.embedder.embed_batch(texts)

        # Store in vector database
        self.vector_store.add_documents(chunks, embeddings)

        return doc.document_id

    def search(
        self,
        query: str,
        top_k: Optional[int] = None,
        filters: Optional[Dict[str, Any]] = None,
    ) -> List[SearchResult]:
        """Search for relevant documents.

        Args:
            query: Search query.
            top_k: Number of results to return. If None, uses config.top_k.
            filters: Optional metadata filters.

        Returns:
            List of SearchResult objects ordered by relevance.

        Raises:
            Various exceptions for search failures.
        """
        # Generate query embedding
        query_embedding = self.embedder.embed_text(query)

        # Search vector store
        k = top_k or self.config.top_k
        results = self.vector_store.search(
            query_embedding=query_embedding,
            top_k=k,
            filters=filters,
        )

        return results

    def retrieve_context(
        self,
        query: str,
        top_k: Optional[int] = None,
    ) -> str:
        """Retrieve and format context for LLM.

        Args:
            query: Search query.
            top_k: Number of results to return. If None, uses config.top_k.

        Returns:
            Formatted context string suitable for LLM input.

        Raises:
            Various exceptions for retrieval failures.
        """
        results = self.search(query, top_k=top_k)

        if not results:
            return ""

        # Format context
        context_parts = []
        for i, result in enumerate(results, 1):
            context_parts.append(f"[{i}] {result.chunk.text}")

        return "\n\n".join(context_parts)

    def clear(self) -> bool:
        """Clear all documents from the knowledge base.

        Returns:
            True if clearing was successful.

        Raises:
            Various exceptions for clearing failures.
        """
        return self.vector_store.clear()



"""Edge case tests for Socratic RAG."""

import pytest
from socratic_rag import RAGClient, RAGConfig
from socratic_rag.models import Document, Chunk
from socratic_rag.exceptions import ChunkingError, EmbeddingError, VectorStoreError


class TestEdgeCases:
    """Test edge cases and boundary conditions."""

    @pytest.fixture
    def client(self):
        """Provide a RAG client."""
        config = RAGConfig(chunk_size=100, chunk_overlap=10)
        client = RAGClient(config)
        yield client
        client.clear()

    # Text Edge Cases
    def test_very_short_document(self, client):
        """Test document with minimal text."""
        doc_id = client.add_document("a", "test.txt")
        assert doc_id is not None

    def test_very_long_document(self, client):
        """Test document with very long text."""
        long_text = "word " * 10000  # ~50KB
        doc_id = client.add_document(long_text, "test.txt")
        assert doc_id is not None
        results = client.search("word")
        assert len(results) > 0

    def test_document_with_special_characters(self, client):
        """Test document with special characters."""
        special_text = "Hello! @#$%^&*() 你好 🎉 日本語"
        doc_id = client.add_document(special_text, "special.txt")
        assert doc_id is not None

    def test_document_with_newlines(self, client):
        """Test document with multiple newlines."""
        multiline = "Line 1\n\n\n\nLine 2\n\n\n\nLine 3"
        doc_id = client.add_document(multiline, "multiline.txt")
        assert doc_id is not None

    def test_whitespace_only_document(self, client):
        """Test document with only whitespace."""
        # Single space should add
        doc_id = client.add_document("   ", "whitespace.txt")
        assert doc_id is not None

    # Search Edge Cases
    def test_search_empty_query(self, client):
        """Test search with empty query."""
        client.add_document("test content", "test.txt")
        # Empty query might be handled differently by embedder
        try:
            results = client.search("")
            # If no error, results should be empty or valid
            assert isinstance(results, list)
        except EmbeddingError:
            # Expected behavior
            pass

    def test_search_very_long_query(self, client):
        """Test search with very long query."""
        client.add_document("content", "test.txt")
        long_query = "word " * 1000
        results = client.search(long_query)
        assert isinstance(results, list)

    def test_search_special_character_query(self, client):
        """Test search with special characters."""
        client.add_document("Hello world", "test.txt")
        results = client.search("!@#$%^&*()")
        assert isinstance(results, list)

    def test_search_with_zero_top_k(self, client):
        """Test search with top_k=0."""
        client.add_document("content", "test.txt")
        with pytest.raises(VectorStoreError):
            client.search("query", top_k=0)

    def test_search_with_negative_top_k(self, client):
        """Test search with negative top_k."""
        client.add_document("content", "test.txt")
        with pytest.raises(VectorStoreError):
            client.search("query", top_k=-5)

    def test_search_with_very_large_top_k(self, client):
        """Test search with very large top_k."""
        client.add_document("content", "test.txt")
        results = client.search("query", top_k=1000000)
        # Should return available results, not error
        assert isinstance(results, list)
        assert len(results) <= 1

    # Configuration Edge Cases
    def test_chunk_size_equals_text_length(self, client):
        """Test when chunk size equals text length."""
        config = RAGConfig(chunk_size=11, chunk_overlap=0)
        client = RAGClient(config)
        doc_id = client.add_document("hello world", "test.txt")
        assert doc_id is not None

    def test_chunk_size_larger_than_text(self, client):
        """Test when chunk size is larger than text."""
        config = RAGConfig(chunk_size=1000, chunk_overlap=0)
        client = RAGClient(config)
        doc_id = client.add_document("short", "test.txt")
        assert doc_id is not None

    def test_zero_chunk_overlap(self, client):
        """Test with zero chunk overlap."""
        config = RAGConfig(chunk_size=50, chunk_overlap=0)
        client = RAGClient(config)
        doc_id = client.add_document("a" * 200, "test.txt")
        assert doc_id is not None

    def test_max_chunk_overlap(self, client):
        """Test with maximum valid chunk overlap."""
        config = RAGConfig(chunk_size=100, chunk_overlap=99)
        client = RAGClient(config)
        doc_id = client.add_document("x" * 500, "test.txt")
        assert doc_id is not None

    # Metadata Edge Cases
    def test_document_with_large_metadata(self, client):
        """Test document with large metadata."""
        large_metadata = {
            f"key_{i}": f"value_{i}" * 100
            for i in range(100)
        }
        doc_id = client.add_document(
            "content",
            "test.txt",
            metadata=large_metadata
        )
        assert doc_id is not None

    def test_document_with_nested_metadata(self, client):
        """Test document with nested metadata."""
        nested_metadata = {
            "level1": {
                "level2": {
                    "level3": "value"
                }
            }
        }
        doc_id = client.add_document(
            "content",
            "test.txt",
            metadata=nested_metadata
        )
        assert doc_id is not None

    def test_document_with_null_metadata_values(self, client):
        """Test document with null metadata values."""
        metadata = {"key1": None, "key2": "value"}
        doc_id = client.add_document(
            "content",
            "test.txt",
            metadata=metadata
        )
        assert doc_id is not None

    # Multiple Operations
    def test_add_many_documents(self, client):
        """Test adding many documents."""
        for i in range(100):
            client.add_document(f"content {i}", f"test{i}.txt")

        results = client.search("content", top_k=10)
        assert len(results) > 0

    def test_search_after_clear(self, client):
        """Test search after clearing knowledge base."""
        client.add_document("content", "test.txt")
        client.clear()

        results = client.search("content")
        assert results == []

    def test_multiple_clears(self, client):
        """Test calling clear multiple times."""
        client.add_document("content", "test.txt")
        assert client.clear()
        assert client.clear()  # Should not error

    # Unicode and Encoding
    def test_unicode_content(self, client):
        """Test with unicode content."""
        unicode_text = "Hello 世界 🌍 مرحبا العالم"
        doc_id = client.add_document(unicode_text, "unicode.txt")
        assert doc_id is not None

        results = client.search("世界")
        assert isinstance(results, list)

    def test_emoji_only_content(self, client):
        """Test with emoji-only content."""
        emoji_text = "🎉 🎊 🎈 🎀 🎁"
        doc_id = client.add_document(emoji_text, "emoji.txt")
        assert doc_id is not None

    # Numerical Edge Cases
    def test_very_small_chunk_size(self, client):
        """Test with very small chunk size."""
        config = RAGConfig(chunk_size=1, chunk_overlap=0)
        client = RAGClient(config)
        doc_id = client.add_document("abc", "test.txt")
        assert doc_id is not None

    # Retrieval Edge Cases
    def test_retrieve_context_no_results(self, client):
        """Test retrieve_context when no documents match."""
        client.add_document("Python content", "python.txt")
        context = client.retrieve_context("completely unrelated query xyz")
        # Should return empty string or minimal context
        assert isinstance(context, str)

    def test_retrieve_context_with_large_top_k(self, client):
        """Test retrieve_context with large top_k."""
        client.add_document("content", "test.txt")
        context = client.retrieve_context("query", top_k=10000)
        assert isinstance(context, str)

    # Concurrent-like Operations
    def test_add_and_search_interleaved(self, client):
        """Test adding and searching interleaved."""
        client.add_document("first", "test1.txt")
        results1 = client.search("first")

        client.add_document("second", "test2.txt")
        results2 = client.search("second")

        assert len(results1) > 0
        assert len(results2) > 0

    def test_same_document_multiple_times(self, client):
        """Test adding same document multiple times."""
        content = "same content"
        id1 = client.add_document(content, "test1.txt")
        id2 = client.add_document(content, "test2.txt")

        # Should have different IDs
        assert id1 != id2

        # Search should find both
        results = client.search("same", top_k=10)
        assert len(results) >= 2



"""Tests for vector store providers."""

import pytest
from socratic_rag.vector_stores import ChromaDBVectorStore
from socratic_rag.models import Chunk
from socratic_rag.exceptions import VectorStoreError


@pytest.mark.unit
class TestChrmaDBVectorStore:
    """Test ChromaDBVectorStore."""

    @pytest.fixture
    def vector_store(self):
        """Provide a vector store instance."""
        store = ChromaDBVectorStore(collection_name="test_collection")
        yield store
        # Cleanup
        store.clear()

    def test_vector_store_initialization(self, vector_store):
        """Test vector store initialization."""
        assert vector_store.collection_name == "test_collection"
        assert vector_store.collection is not None

    def test_add_documents(self, vector_store):
        """Test adding documents to vector store."""
        chunks = [
            Chunk.create(
                text="Hello world",
                document_id="doc1",
            ),
            Chunk.create(
                text="Test document",
                document_id="doc1",
            ),
        ]
        embeddings = [
            [0.1] * 384,  # 384-dimensional embedding
            [0.2] * 384,
        ]

        chunk_ids = vector_store.add_documents(chunks, embeddings)
        assert len(chunk_ids) == 2
        assert all(isinstance(id, str) for id in chunk_ids)

    def test_add_empty_documents(self, vector_store):
        """Test adding empty document list."""
        chunk_ids = vector_store.add_documents([], [])
        assert chunk_ids == []

    def test_mismatched_chunks_embeddings(self, vector_store):
        """Test that mismatched chunks and embeddings raise error."""
        chunks = [
            Chunk.create(text="Hello", document_id="doc1"),
        ]
        embeddings = [
            [0.1] * 384,
            [0.2] * 384,
        ]

        with pytest.raises(VectorStoreError):
            vector_store.add_documents(chunks, embeddings)

    def test_search(self, vector_store):
        """Test searching documents."""
        chunks = [
            Chunk.create(
                text="Python is a programming language",
                document_id="doc1",
            ),
            Chunk.create(
                text="Java is another programming language",
                document_id="doc2",
            ),
        ]
        embeddings = [
            [0.1] * 384,
            [0.2] * 384,
        ]

        vector_store.add_documents(chunks, embeddings)

        # Search with similar embedding to first chunk
        query_embedding = [0.11] * 384
        results = vector_store.search(query_embedding, top_k=2)

        assert len(results) <= 2
        assert all(hasattr(r, "chunk") and hasattr(r, "score") for r in results)

    def test_search_empty_store(self, vector_store):
        """Test searching empty store."""
        query_embedding = [0.1] * 384
        results = vector_store.search(query_embedding, top_k=5)
        assert results == []

    def test_delete_documents(self, vector_store):
        """Test deleting documents."""
        chunks = [
            Chunk.create(text="Hello", document_id="doc1"),
            Chunk.create(text="World", document_id="doc1"),
        ]
        embeddings = [
            [0.1] * 384,
            [0.2] * 384,
        ]

        chunk_ids = vector_store.add_documents(chunks, embeddings)
        assert vector_store.delete(chunk_ids)

    def test_delete_empty_list(self, vector_store):
        """Test deleting empty list."""
        assert vector_store.delete([])

    def test_get_document(self, vector_store):
        """Test getting document by ID."""
        chunk = Chunk.create(
            text="Hello world",
            document_id="doc1",
            metadata={"key": "value"},
        )
        embeddings = [[0.1] * 384]

        chunk_ids = vector_store.add_documents([chunk], embeddings)
        chunk_id = chunk_ids[0]

        retrieved = vector_store.get(chunk_id)
        assert retrieved is not None
        assert retrieved.text == "Hello world"
        assert retrieved.document_id == "doc1"

    def test_get_nonexistent_document(self, vector_store):
        """Test getting nonexistent document."""
        retrieved = vector_store.get("nonexistent_id")
        assert retrieved is None

    def test_clear(self, vector_store):
        """Test clearing vector store."""
        chunks = [
            Chunk.create(text="Hello", document_id="doc1"),
        ]
        embeddings = [[0.1] * 384]

        vector_store.add_documents(chunks, embeddings)
        assert vector_store.clear()

        # After clearing, search should return empty
        results = vector_store.search([0.1] * 384, top_k=5)
        assert results == []



"""Custom exceptions for Socratic RAG."""


class SocraticRAGError(Exception):
    """Base exception for Socratic RAG."""

    pass


class ConfigurationError(SocraticRAGError):
    """Raised when configuration is invalid."""

    pass


class VectorStoreError(SocraticRAGError):
    """Raised when vector store operation fails."""

    pass


class EmbeddingError(SocraticRAGError):
    """Raised when embedding operation fails."""

    pass


class ChunkingError(SocraticRAGError):
    """Raised when chunking operation fails."""

    pass


class ProcessorError(SocraticRAGError):
    """Raised when document processing fails."""

    pass


class DocumentNotFoundError(VectorStoreError):
    """Raised when document is not found."""

    pass


class ProviderNotFoundError(ConfigurationError):
    """Raised when provider is not found."""

    pass


class InvalidProviderError(ConfigurationError):
    """Raised when provider configuration is invalid."""

    pass


class AsyncRAGError(SocraticRAGError):
    """Raised when async operation fails."""

    pass



"""Tests for document processors."""

import tempfile
from pathlib import Path
import pytest
from socratic_rag.processors import TextProcessor, MarkdownProcessor, PDFProcessor
from socratic_rag.exceptions import ProcessorError


class TestTextProcessor:
    """Test TextProcessor."""

    @pytest.fixture
    def processor(self):
        """Provide a text processor."""
        return TextProcessor()

    def test_process_text_file(self, processor):
        """Test processing a text file."""
        with tempfile.NamedTemporaryFile(
            mode="w", suffix=".txt", delete=False, encoding="utf-8"
        ) as f:
            f.write("This is test content.")
            f.flush()
            temp_path = f.name

        try:
            documents = processor.process(temp_path)
            assert len(documents) == 1
            assert documents[0].content == "This is test content."
            assert documents[0].source == temp_path
        finally:
            Path(temp_path).unlink()

    def test_process_nonexistent_file(self, processor):
        """Test processing nonexistent file."""
        with pytest.raises(ProcessorError):
            processor.process("nonexistent_file.txt")

    def test_process_empty_file(self, processor):
        """Test processing empty file."""
        with tempfile.NamedTemporaryFile(
            mode="w", suffix=".txt", delete=False, encoding="utf-8"
        ) as f:
            f.flush()
            temp_path = f.name

        try:
            with pytest.raises(ProcessorError):
                processor.process(temp_path)
        finally:
            Path(temp_path).unlink()


class TestMarkdownProcessor:
    """Test MarkdownProcessor."""

    @pytest.fixture
    def processor(self):
        """Provide a markdown processor."""
        return MarkdownProcessor()

    def test_process_markdown_file(self, processor):
        """Test processing a markdown file."""
        content = "# Hello\n\nThis is markdown content."

        with tempfile.NamedTemporaryFile(
            mode="w", suffix=".md", delete=False, encoding="utf-8"
        ) as f:
            f.write(content)
            f.flush()
            temp_path = f.name

        try:
            documents = processor.process(temp_path)
            assert len(documents) == 1
            assert documents[0].content == content
        finally:
            Path(temp_path).unlink()

    def test_process_nonexistent_file(self, processor):
        """Test processing nonexistent markdown file."""
        with pytest.raises(ProcessorError):
            processor.process("nonexistent_file.md")



"""Async RAG client interface."""

import asyncio
from typing import Any, Dict, List, Optional
from .models import Document, RAGConfig, SearchResult
from .client import RAGClient


class AsyncRAGClient:
    """Async RAG client interface.

    Provides async/await interface for RAG operations.
    Wraps the synchronous RAGClient for async usage.
    """

    def __init__(self, config: Optional[RAGConfig] = None) -> None:
        """Initialize async RAG client.

        Args:
            config: RAGConfig object. If None, uses default configuration.

        Raises:
            ConfigurationError: If configuration is invalid.
        """
        self.client = RAGClient(config)
        self.config = self.client.config

    async def add_document(
        self,
        content: str,
        source: str,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> str:
        """Add document to knowledge base asynchronously.

        Args:
            content: Document content.
            source: Document source (e.g., filename, URL).
            metadata: Optional metadata to attach to document.

        Returns:
            Document ID.

        Raises:
            Various exceptions for processing failures.
        """
        return await asyncio.to_thread(
            self.client.add_document,
            content,
            source,
            metadata,
        )

    async def search(
        self,
        query: str,
        top_k: Optional[int] = None,
        filters: Optional[Dict[str, Any]] = None,
    ) -> List[SearchResult]:
        """Search for relevant documents asynchronously.

        Args:
            query: Search query.
            top_k: Number of results to return. If None, uses config.top_k.
            filters: Optional metadata filters.

        Returns:
            List of SearchResult objects ordered by relevance.

        Raises:
            Various exceptions for search failures.
        """
        return await asyncio.to_thread(
            self.client.search,
            query,
            top_k,
            filters,
        )

    async def retrieve_context(
        self,
        query: str,
        top_k: Optional[int] = None,
    ) -> str:
        """Retrieve and format context for LLM asynchronously.

        Args:
            query: Search query.
            top_k: Number of results to return. If None, uses config.top_k.

        Returns:
            Formatted context string suitable for LLM input.

        Raises:
            Various exceptions for retrieval failures.
        """
        return await asyncio.to_thread(
            self.client.retrieve_context,
            query,
            top_k,
        )

    async def clear(self) -> bool:
        """Clear all documents from the knowledge base asynchronously.

        Returns:
            True if clearing was successful.

        Raises:
            Various exceptions for clearing failures.
        """
        return await asyncio.to_thread(self.client.clear)



"""Tests for RAG client."""

import pytest
from socratic_rag import RAGClient, RAGConfig, AsyncRAGClient
from socratic_rag.exceptions import ProviderNotFoundError


class TestRAGClient:
    """Test RAGClient."""

    @pytest.fixture
    def client(self):
        """Provide a RAG client."""
        config = RAGConfig(
            chunk_size=100,
            chunk_overlap=10,
        )
        client = RAGClient(config)
        yield client
        client.clear()

    def test_client_initialization(self, client):
        """Test client initialization."""
        assert client.config is not None
        assert client.config.vector_store == "chromadb"

    def test_add_document(self, client):
        """Test adding a document."""
        doc_id = client.add_document(
            content="Python is a programming language.",
            source="test.txt",
        )
        assert doc_id is not None
        assert isinstance(doc_id, str)

    def test_add_document_with_metadata(self, client):
        """Test adding document with metadata."""
        doc_id = client.add_document(
            content="Test content",
            source="test.txt",
            metadata={"author": "John", "date": "2024"},
        )
        assert doc_id is not None

    def test_search(self, client):
        """Test searching documents."""
        # Add documents
        client.add_document("Python is great", "test1.txt")
        client.add_document("Java is powerful", "test2.txt")

        # Search
        results = client.search("Python", top_k=5)
        assert len(results) > 0
        assert all(hasattr(r, "chunk") and hasattr(r, "score") for r in results)

    def test_search_empty_index(self, client):
        """Test searching empty index."""
        results = client.search("test")
        assert results == []

    def test_retrieve_context(self, client):
        """Test retrieving context."""
        client.add_document("Python is great", "test1.txt")
        client.add_document("Java is powerful", "test2.txt")

        context = client.retrieve_context("Python")
        assert isinstance(context, str)
        assert len(context) > 0

    def test_retrieve_context_empty(self, client):
        """Test retrieving context from empty index."""
        context = client.retrieve_context("test")
        assert context == ""

    def test_clear(self, client):
        """Test clearing the knowledge base."""
        client.add_document("Test content", "test.txt")
        assert client.clear()

        # After clearing, search should return nothing
        results = client.search("test")
        assert results == []

    def test_invalid_vector_store(self):
        """Test invalid vector store provider."""
        config = RAGConfig(vector_store="invalid_store")
        client = RAGClient(config)
        with pytest.raises(ProviderNotFoundError):
            _ = client.vector_store

    def test_invalid_embedder(self):
        """Test invalid embedder provider."""
        config = RAGConfig(embedder="invalid_embedder")
        client = RAGClient(config)
        with pytest.raises(ProviderNotFoundError):
            _ = client.embedder

    def test_invalid_chunker(self):
        """Test invalid chunker provider."""
        config = RAGConfig(chunking_strategy="invalid_chunker")
        client = RAGClient(config)
        with pytest.raises(ProviderNotFoundError):
            _ = client.chunker

    def test_multiple_documents(self, client):
        """Test adding and searching multiple documents."""
        documents = [
            ("Machine learning is a subset of AI", "ml.txt"),
            ("Deep learning uses neural networks", "dl.txt"),
            ("Natural language processing is important", "nlp.txt"),
        ]

        for content, source in documents:
            client.add_document(content, source)

        results = client.search("neural networks")
        assert len(results) > 0

    def test_search_with_top_k(self, client):
        """Test search with custom top_k."""
        for i in range(10):
            client.add_document(f"Document {i} about testing", f"test{i}.txt")

        results = client.search("testing", top_k=3)
        assert len(results) <= 3

    def test_default_config(self):
        """Test client with default config."""
        client = RAGClient()
        assert client.config.chunk_size == 512
        assert client.config.top_k == 5


class TestAsyncRAGClient:
    """Test AsyncRAGClient."""

    @pytest.fixture
    def async_client(self):
        """Provide an async RAG client."""
        config = RAGConfig(chunk_size=100)
        client = AsyncRAGClient(config)
        yield client
        client.client.clear()

    @pytest.mark.asyncio
    async def test_async_add_document(self, async_client):
        """Test async document addition."""
        doc_id = await async_client.add_document(
            content="Test content",
            source="test.txt",
        )
        assert doc_id is not None

    @pytest.mark.asyncio
    async def test_async_search(self, async_client):
        """Test async search."""
        await async_client.add_document("Python is great", "test1.txt")
        await async_client.add_document("Java is powerful", "test2.txt")

        results = await async_client.search("Python")
        assert isinstance(results, list)

    @pytest.mark.asyncio
    async def test_async_retrieve_context(self, async_client):
        """Test async context retrieval."""
        await async_client.add_document("Test content", "test.txt")
        context = await async_client.retrieve_context("test")
        assert isinstance(context, str)

    @pytest.mark.asyncio
    async def test_async_clear(self, async_client):
        """Test async clearing."""
        await async_client.add_document("Test", "test.txt")
        result = await async_client.clear()
        assert result



"""Tests for LLM-powered RAG."""

import pytest
from socratic_rag import RAGClient, RAGConfig, LLMPoweredRAG
from socratic_rag.exceptions import SocraticRAGError


class MockLLMClient:
    """Mock LLM client for testing."""

    def __init__(self, response_text: str = "Test response"):
        self.response_text = response_text
        self.last_prompt = None

    def chat(self, prompt: str, **kwargs):
        """Return mock response."""
        self.last_prompt = prompt

        class Response:
            def __init__(self, content):
                self.content = content

        return Response(self.response_text)


class TestLLMPoweredRAG:
    """Test LLMPoweredRAG."""

    @pytest.fixture
    def rag_client(self):
        """Provide a RAG client."""
        config = RAGConfig(chunk_size=100)
        client = RAGClient(config)
        yield client
        client.clear()

    @pytest.fixture
    def llm_client(self):
        """Provide a mock LLM client."""
        return MockLLMClient(response_text="This is a test answer.")

    @pytest.fixture
    def llm_rag(self, rag_client, llm_client):
        """Provide LLM-powered RAG."""
        return LLMPoweredRAG(rag_client=rag_client, llm_client=llm_client)

    def test_initialization(self, llm_rag):
        """Test LLMPoweredRAG initialization."""
        assert llm_rag.rag is not None
        assert llm_rag.llm is not None

    def test_initialization_without_llm(self, rag_client):
        """Test initialization without LLM."""
        llm_rag = LLMPoweredRAG(rag_client=rag_client, llm_client=None)
        assert llm_rag.rag is not None
        assert llm_rag.llm is None

    def test_add_document(self, llm_rag):
        """Test adding document."""
        doc_id = llm_rag.add_document(
            content="Test content",
            source="test.txt",
        )
        assert doc_id is not None

    def test_search(self, llm_rag):
        """Test search."""
        llm_rag.add_document("Test content", "test.txt")
        results = llm_rag.search("test")
        assert isinstance(results, list)

    def test_retrieve_context(self, llm_rag):
        """Test context retrieval."""
        llm_rag.add_document("Test content", "test.txt")
        context = llm_rag.retrieve_context("test")
        assert isinstance(context, str)

    def test_generate_answer(self, llm_rag):
        """Test answer generation."""
        llm_rag.add_document("Python is great", "test.txt")

        answer = llm_rag.generate_answer("What is Python?")
        assert isinstance(answer, str)
        assert len(answer) > 0

    def test_generate_answer_without_llm(self, rag_client):
        """Test that answer generation fails without LLM."""
        llm_rag = LLMPoweredRAG(rag_client=rag_client, llm_client=None)
        llm_rag.add_document("Test content", "test.txt")

        with pytest.raises(SocraticRAGError):
            llm_rag.generate_answer("test")

    def test_generate_answer_with_custom_prompt(self, llm_rag):
        """Test answer generation with custom system prompt."""
        llm_rag.add_document("Python is great", "test.txt")

        answer = llm_rag.generate_answer(
            "What is Python?",
            system_prompt="You are a Python expert.",
        )
        assert isinstance(answer, str)
        assert "You are a Python expert" in llm_rag.llm.last_prompt

    def test_clear(self, llm_rag):
        """Test clearing knowledge base."""
        llm_rag.add_document("Test content", "test.txt")
        assert llm_rag.clear()

    def test_custom_context_prefix_and_separator(self, llm_rag):
        """Test custom context prefix and separator."""
        llm_rag.add_document("Content 1", "test1.txt")
        llm_rag.add_document("Content 2", "test2.txt")

        answer = llm_rag.generate_answer(
            "test",
            context_prefix="Documents:\n",
            context_separator=" | ",
        )
        assert "Documents:" in llm_rag.llm.last_prompt

    def test_answer_contains_context(self, llm_rag):
        """Test that generated answer incorporates context."""
        llm_rag.add_document("Important fact about Python", "test.txt")

        _ = llm_rag.generate_answer("Python question")

        # The prompt sent to LLM should contain the context
        assert "Important fact about Python" in llm_rag.llm.last_prompt



"""Tests for integrations."""

import pytest
from socratic_rag import RAGClient, RAGConfig
from socratic_rag.integrations.openclaw import SocraticRAGSkill
from socratic_rag.integrations.langchain import SocraticRAGRetriever


class TestOpenclaw:
    """Test Openclaw RAG skill."""

    @pytest.fixture
    def skill(self):
        """Provide a RAG skill."""
        skill = SocraticRAGSkill(
            vector_store="chromadb",
            chunk_size=100,
        )
        yield skill
        skill.clear()

    def test_skill_initialization(self, skill):
        """Test skill initialization."""
        assert skill.client is not None
        assert skill.client.config.vector_store == "chromadb"

    def test_add_document(self, skill):
        """Test adding document through skill."""
        doc_id = skill.add_document(
            content="Test content",
            source="test.txt",
        )
        assert doc_id is not None

    def test_search(self, skill):
        """Test search through skill."""
        skill.add_document("Python is great", "test1.txt")
        skill.add_document("Java is powerful", "test2.txt")

        results = skill.search("Python")
        assert isinstance(results, list)
        assert all("score" in r and "text" in r for r in results)

    def test_retrieve_context(self, skill):
        """Test context retrieval through skill."""
        skill.add_document("Test content", "test.txt")
        context = skill.retrieve_context("test")
        assert isinstance(context, str)

    def test_get_config(self, skill):
        """Test getting config from skill."""
        config = skill.get_config()
        assert config.vector_store == "chromadb"
        assert config.chunk_size == 100


class TestLangChain:
    """Test LangChain retriever."""

    @pytest.fixture
    def client(self):
        """Provide a RAG client."""
        config = RAGConfig(chunk_size=100)
        client = RAGClient(config)
        yield client
        client.clear()

    @pytest.fixture
    def retriever(self, client):
        """Provide a LangChain retriever."""
        return SocraticRAGRetriever(client=client, top_k=5)

    def test_retriever_initialization(self, retriever):
        """Test retriever initialization."""
        assert retriever.client is not None
        assert retriever.top_k == 5

    def test_get_relevant_documents(self, client, retriever):
        """Test getting relevant documents."""
        client.add_document("Python is great", "test1.txt")
        client.add_document("Java is powerful", "test2.txt")

        documents = retriever.get_relevant_documents("Python")
        assert isinstance(documents, list)
        assert all(hasattr(d, "page_content") for d in documents)

    def test_documents_have_metadata(self, client, retriever):
        """Test that retrieved documents have metadata."""
        client.add_document("Test content", "test.txt")

        documents = retriever.get_relevant_documents("test")
        assert len(documents) > 0
        assert "chunk_id" in documents[0].metadata
        assert "score" in documents[0].metadata



"""Data models for Socratic RAG."""

from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional
from datetime import datetime
import uuid


@dataclass
class Chunk:
    """Text chunk with metadata."""

    text: str
    chunk_id: str
    document_id: str
    metadata: Dict[str, Any] = field(default_factory=dict)
    start_char: int = 0
    end_char: int = 0

    @classmethod
    def create(
        cls,
        text: str,
        document_id: str,
        metadata: Optional[Dict[str, Any]] = None,
        start_char: int = 0,
        end_char: int = 0,
    ) -> "Chunk":
        """Create a chunk with auto-generated ID."""
        return cls(
            text=text,
            chunk_id=str(uuid.uuid4()),
            document_id=document_id,
            metadata=metadata or {},
            start_char=start_char,
            end_char=end_char,
        )


@dataclass
class Document:
    """Document to be indexed."""

    content: str
    document_id: str
    source: str
    metadata: Dict[str, Any] = field(default_factory=dict)
    created_at: datetime = field(default_factory=datetime.utcnow)

    @classmethod
    def create(
        cls,
        content: str,
        source: str,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> "Document":
        """Create a document with auto-generated ID."""
        return cls(
            content=content,
            document_id=str(uuid.uuid4()),
            source=source,
            metadata=metadata or {},
            created_at=datetime.utcnow(),
        )


@dataclass
class SearchResult:
    """Search result with score."""

    chunk: Chunk
    score: float
    document: Optional[Document] = None


@dataclass
class RAGConfig:
    """Configuration for RAG client."""

    vector_store: str = "chromadb"
    embedder: str = "sentence-transformers"
    chunking_strategy: str = "fixed"
    chunk_size: int = 512
    chunk_overlap: int = 50
    top_k: int = 5
    embedding_cache: bool = True
    cache_ttl: int = 3600
    collection_name: str = "socratic_rag"

    def __post_init__(self) -> None:
        """Validate configuration."""
        if self.chunk_size <= 0:
            raise ValueError("chunk_size must be positive")
        if self.chunk_overlap >= self.chunk_size:
            raise ValueError("chunk_overlap must be less than chunk_size")
        if self.top_k <= 0:
            raise ValueError("top_k must be positive")
        if self.cache_ttl <= 0:
            raise ValueError("cache_ttl must be positive")



"""Tests for embedding providers."""

import pytest
from socratic_rag.embeddings import SentenceTransformersEmbedder
from socratic_rag.exceptions import EmbeddingError


class TestSentenceTransformersEmbedder:
    """Test SentenceTransformersEmbedder."""

    @pytest.fixture
    def embedder(self):
        """Provide an embedder instance."""
        return SentenceTransformersEmbedder()

    def test_embedder_initialization(self, embedder):
        """Test embedder initialization."""
        assert embedder.model is not None
        assert embedder.dimension > 0

    def test_embed_text(self, embedder):
        """Test embedding a single text."""
        embedding = embedder.embed_text("Hello world")
        assert isinstance(embedding, list)
        assert len(embedding) == embedder.dimension
        assert all(isinstance(x, float) for x in embedding)

    def test_embed_batch(self, embedder):
        """Test embedding a batch of texts."""
        texts = ["Hello", "World", "Test"]
        embeddings = embedder.embed_batch(texts)
        assert isinstance(embeddings, list)
        assert len(embeddings) == len(texts)
        assert all(len(e) == embedder.dimension for e in embeddings)

    def test_embed_empty_text(self, embedder):
        """Test that embedding empty text raises error."""
        with pytest.raises(EmbeddingError):
            embedder.embed_text("")

    def test_embed_empty_batch(self, embedder):
        """Test that embedding empty batch raises error."""
        with pytest.raises(EmbeddingError):
            embedder.embed_batch([])

    def test_embed_similar_texts(self, embedder):
        """Test that similar texts have similar embeddings."""
        text1 = "The cat sat on the mat"
        text2 = "A cat sat on a mat"
        text3 = "The dog ran in the park"

        emb1 = embedder.embed_text(text1)
        emb2 = embedder.embed_text(text2)
        emb3 = embedder.embed_text(text3)

        # Calculate cosine similarity
        def cosine_similarity(a, b):
            dot_product = sum(x * y for x, y in zip(a, b))
            norm_a = sum(x**2 for x in a) ** 0.5
            norm_b = sum(x**2 for x in b) ** 0.5
            return dot_product / (norm_a * norm_b)

        similarity_12 = cosine_similarity(emb1, emb2)
        similarity_13 = cosine_similarity(emb1, emb3)

        # Similar texts should have higher similarity
        assert similarity_12 > similarity_13

    def test_embedding_dimension(self, embedder):
        """Test that embedding dimension is consistent."""
        dim = embedder.dimension
        assert dim == 384  # all-MiniLM-L6-v2 has 384 dimensions

    def test_custom_model(self):
        """Test embedder with custom model."""
        embedder = SentenceTransformersEmbedder(
            model_name="all-MiniLM-L6-v2"
        )
        assert embedder.model_name == "all-MiniLM-L6-v2"
        assert embedder.dimension > 0



"""Tests for data models."""

import pytest
from datetime import datetime
from socratic_rag.models import Chunk, Document, RAGConfig, SearchResult


class TestChunk:
    """Test Chunk model."""

    def test_chunk_creation(self):
        """Test creating a chunk."""
        chunk = Chunk(
            text="Hello world",
            chunk_id="id1",
            document_id="doc1",
            metadata={"key": "value"},
        )
        assert chunk.text == "Hello world"
        assert chunk.chunk_id == "id1"
        assert chunk.document_id == "doc1"
        assert chunk.metadata == {"key": "value"}

    def test_chunk_create_with_auto_id(self):
        """Test creating a chunk with auto-generated ID."""
        chunk = Chunk.create(
            text="Hello world",
            document_id="doc1",
            metadata={"key": "value"},
        )
        assert chunk.text == "Hello world"
        assert chunk.document_id == "doc1"
        assert chunk.chunk_id is not None
        assert len(chunk.chunk_id) > 0

    def test_chunk_default_metadata(self):
        """Test chunk with default metadata."""
        chunk = Chunk.create(
            text="Hello",
            document_id="doc1",
        )
        assert chunk.metadata == {}


class TestDocument:
    """Test Document model."""

    def test_document_creation(self):
        """Test creating a document."""
        doc = Document(
            content="Hello world",
            document_id="doc1",
            source="test.txt",
            metadata={"key": "value"},
        )
        assert doc.content == "Hello world"
        assert doc.document_id == "doc1"
        assert doc.source == "test.txt"
        assert doc.metadata == {"key": "value"}

    def test_document_create_with_auto_id(self):
        """Test creating a document with auto-generated ID."""
        doc = Document.create(
            content="Hello world",
            source="test.txt",
        )
        assert doc.content == "Hello world"
        assert doc.source == "test.txt"
        assert doc.document_id is not None
        assert len(doc.document_id) > 0

    def test_document_created_at(self):
        """Test document created_at timestamp."""
        doc = Document.create(
            content="Hello",
            source="test.txt",
        )
        assert isinstance(doc.created_at, datetime)


class TestSearchResult:
    """Test SearchResult model."""

    def test_search_result_creation(self):
        """Test creating a search result."""
        chunk = Chunk.create(
            text="Hello",
            document_id="doc1",
        )
        result = SearchResult(chunk=chunk, score=0.95)
        assert result.chunk == chunk
        assert result.score == 0.95
        assert result.document is None

    def test_search_result_with_document(self):
        """Test search result with document."""
        chunk = Chunk.create(
            text="Hello",
            document_id="doc1",
        )
        doc = Document.create(
            content="Hello world",
            source="test.txt",
        )
        result = SearchResult(chunk=chunk, score=0.95, document=doc)
        assert result.chunk == chunk
        assert result.score == 0.95
        assert result.document == doc


class TestRAGConfig:
    """Test RAGConfig model."""

    def test_default_config(self):
        """Test default configuration."""
        config = RAGConfig()
        assert config.vector_store == "chromadb"
        assert config.embedder == "sentence-transformers"
        assert config.chunking_strategy == "fixed"
        assert config.chunk_size == 512
        assert config.chunk_overlap == 50
        assert config.top_k == 5

    def test_custom_config(self):
        """Test custom configuration."""
        config = RAGConfig(
            vector_store="qdrant",
            chunk_size=256,
            chunk_overlap=25,
        )
        assert config.vector_store == "qdrant"
        assert config.chunk_size == 256
        assert config.chunk_overlap == 25

    def test_invalid_chunk_size(self):
        """Test that negative chunk_size raises error."""
        with pytest.raises(ValueError, match="chunk_size must be positive"):
            RAGConfig(chunk_size=-1)

    def test_overlap_greater_than_chunk_size(self):
        """Test that overlap >= chunk_size raises error."""
        with pytest.raises(ValueError, match="chunk_overlap must be less"):
            RAGConfig(chunk_size=100, chunk_overlap=100)

    def test_invalid_top_k(self):
        """Test that negative top_k raises error."""
        with pytest.raises(ValueError, match="top_k must be positive"):
            RAGConfig(top_k=-1)

    def test_invalid_cache_ttl(self):
        """Test that negative cache_ttl raises error."""
        with pytest.raises(ValueError, match="cache_ttl must be positive"):
            RAGConfig(cache_ttl=-1)



"""LLM-powered RAG client using Socrates Nexus integration."""

from typing import Optional, Any, Dict
from .client import RAGClient
from .exceptions import SocraticRAGError


class LLMPoweredRAG:
    """RAG with LLM for answer generation.

    Combines document retrieval with LLM-powered answer generation
    using Socrates Nexus LLM client.
    """

    def __init__(
        self,
        rag_client: RAGClient,
        llm_client: Optional[Any] = None,
    ) -> None:
        """Initialize LLM-powered RAG.

        Args:
            rag_client: RAGClient instance for retrieval.
            llm_client: Socrates Nexus LLMClient instance.
                If None, LLM features will not be available.

        Raises:
            SocraticRAGError: If initialization fails.
        """
        self.rag = rag_client
        self.llm = llm_client

    def generate_answer(
        self,
        query: str,
        top_k: Optional[int] = None,
        context_prefix: str = "Context:\n",
        context_separator: str = "\n\n",
        system_prompt: Optional[str] = None,
        **llm_kwargs: Any,
    ) -> str:
        """Generate answer using RAG + LLM.

        Retrieves relevant documents and uses LLM to generate an answer
        based on the retrieved context.

        Args:
            query: Question to answer.
            top_k: Number of context documents to retrieve.
            context_prefix: Prefix for context section.
            context_separator: Separator between context documents.
            system_prompt: Custom system prompt for the LLM.
            **llm_kwargs: Additional arguments passed to LLM.

        Returns:
            Generated answer string.

        Raises:
            SocraticRAGError: If LLM is not configured or generation fails.
        """
        if not self.llm:
            raise SocraticRAGError(
                "LLM client is not configured. "
                "Provide llm_client when initializing LLMPoweredRAG."
            )

        # Retrieve context
        context = self.rag.retrieve_context(query, top_k=top_k)

        # Build prompt
        if not context:
            context = "No relevant documents found."

        prompt_parts = [context_prefix + context]

        if system_prompt:
            full_prompt = f"{system_prompt}\n\n" + context_separator.join(prompt_parts)
        else:
            full_prompt = context_separator.join(prompt_parts)

        full_prompt += f"\n\nQuestion: {query}\n\nAnswer:"

        # Generate answer with LLM
        try:
            response = self.llm.chat(full_prompt, **llm_kwargs)
            return response.content if hasattr(response, "content") else str(response)
        except AttributeError as e:
            raise SocraticRAGError(
                f"LLM client does not have expected interface. "
                f"Ensure it's a Socrates Nexus LLMClient. Error: {e}"
            )
        except Exception as e:
            raise SocraticRAGError(f"Failed to generate answer: {e}")

    def retrieve_context(
        self,
        query: str,
        top_k: Optional[int] = None,
    ) -> str:
        """Retrieve context for a query.

        Args:
            query: Query string.
            top_k: Number of results to retrieve.

        Returns:
            Formatted context string.
        """
        return self.rag.retrieve_context(query, top_k=top_k)

    def search(
        self,
        query: str,
        top_k: Optional[int] = None,
        filters: Optional[Dict[str, Any]] = None,
    ) -> list:
        """Search for documents.

        Args:
            query: Search query.
            top_k: Number of results to return.
            filters: Optional metadata filters.

        Returns:
            List of SearchResult objects.
        """
        return self.rag.search(query, top_k=top_k, filters=filters)

    def add_document(
        self,
        content: str,
        source: str,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> str:
        """Add document to knowledge base.

        Args:
            content: Document content.
            source: Document source.
            metadata: Optional metadata.

        Returns:
            Document ID.
        """
        return self.rag.add_document(content, source, metadata)

    def clear(self) -> bool:
        """Clear knowledge base.

        Returns:
            True if successful.
        """
        return self.rag.clear()
